#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: savechanges.pl,v 1.0.0.0 Fri 20 Apr 2007 03:04:24 AM CEST mstauber Exp $
# Copyright 2006-2007 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# Handler which writes / updates the drbd and HA config files.
#

use CCE;
use POSIX qw(isalpha);

my $DEBUG = 0;

my $cce = new CCE('Domain' => 'vz-vserver');

if ($DEBUG == 0) {
    $cce->connectfd();
}
else {
    $cce->connectuds(); # only for debugging
}

$drbd_conf = '/etc/drbd.conf';
$ha_conf = '/etc/ha.d/ha.cf';

$cluster = $cce->event_object();
$cluster_new = $cce->event_new();
$cluster_old = $cce->event_old();

($sys_oid) = $cce->find('Cluster', '');
($ok, $system) = $cce->get($sys_oid);

# Do the deeds:
&use_gui_input;

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);

# Use GUI submitted new individual values:
sub use_gui_input {

	vps_write_conf(
    	    $system->{'degr_wfc_timeout'},
    	    $system->{'rate'},
    	    $system->{'Std_Pri_fqdn'},
    	    $system->{'Std_Pri_Device'},
    	    $system->{'Std_Pri_Disk'},
    	    $system->{'Std_Pri_ipAddr'},
    	    $system->{'Std_Sec_fqdn'},
    	    $system->{'Std_Sec_Device'},
    	    $system->{'Std_Sec_Disk'},
    	    $system->{'Std_Sec_ipAddr'},

    	    $system->{'keepalive'},
    	    $system->{'warntime'},
   	    $system->{'deadtime'},
    	    $system->{'initdead'},
    	    $system->{'ucast_eth1_master'},
    	    $system->{'ucast_eth1_slave'},
    	    $system->{'ucast_eth0_master'},
    	    $system->{'ucast_eth0_slave'},
    	    $system->{'auto_failback'},
    	    $system->{'ping'}
        );
}

sub vps_write_conf {

        my $degr_wfc_timeout   	= shift;
        my $rate            	= shift;
        my $Std_Pri_fqdn       	= shift;
        my $Std_Pri_Device     	= shift;
        my $Std_Pri_Disk       	= shift;
        my $Std_Pri_ipAddr      = shift;
        my $Std_Sec_fqdn        = shift;
        my $Std_Sec_Device      = shift;
        my $Std_Sec_Disk        = shift;
        my $Std_Sec_ipAddr      = shift;

        my $keepalive           = shift;
        my $warntime            = shift;
        my $deadtime            = shift;
        my $initdead            = shift;
        my $ucast_eth1_master   = shift;
        my $ucast_eth1_slave    = shift;
        my $ucast_eth0_master   = shift;
        my $ucast_eth0_slave    = shift;
        my $auto_failback       = shift;
        my $ping                = shift;


        open(CONF, ">$drbd_conf") || die "Can't write to $drbd_conf!";
        print CONF "#
# please have a a look at the example configuration file in
# /usr/share/doc/drbd/drbd.conf
#

resource vz {
  protocol C;
  incon-degr-cmd \"echo 'DRBD: Primary on master1 degraded!' | wall ; sleep 60 ; halt -f\";

  startup {
    degr-wfc-timeout $degr_wfc_timeout;
  }

  net {
    on-disconnect reconnect;
  }

  disk {
    on-io-error   detach;
  }

  syncer {
    rate $rate;
    group 1;
    al-extents 257;
  }

  on $Std_Pri_fqdn {
    device     $Std_Pri_Device;
    disk       $Std_Pri_Disk;
    address    $Std_Pri_ipAddr:7788;
    meta-disk  internal;
  }

  on $Std_Sec_fqdn {
    device     $Std_Sec_Device;
    disk       $Std_Sec_Disk;
    address    $Std_Sec_ipAddr:7788;
    meta-disk  internal;
  }

}
";
	close CONF;

	open(CONF, ">$ha_conf") || die "Can't write to $drbd_conf!";
	print CONF "#
# Heartbeat configuration:
#

logfacility daemon

# Heartbeat cluster members
node $Std_Pri_fqdn
node $Std_Sec_fqdn

# Heartbeat communication timing
keepalive $keepalive
warntime $warntime
deadtime $deadtime
initdead $initdead

# Heartbeat communication paths
udpport 694
ucast eth1 $ucast_eth1_master
ucast eth1 $ucast_eth1_slave
ucast eth0 $ucast_eth0_master
ucast eth0 $ucast_eth0_slave
#baud 19200
#serial /dev/ttyS1

# Fail back automatically
auto_failback $auto_failback

# Monitoring of network connection to default gateway
ping $ping
respawn hacluster /usr/lib64/heartbeat/ipfail
";
	close CONF;


    # Restart Daemons:
    system("/etc/init.d/drbd reload");
    system("/etc/init.d/heartbeat reload");
	
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


