#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: update_cluster_info.pl, v1.1.0.0 Thu 19 Apr 2007 05:45:20 AM CEST mstauber Exp $
# Copyright 2006-2007 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.

# This constructor / handler polls /etc/drbd.conf and /proc/drbd for information
# about the cluster configuration and its status. That info is then written into
# CCE for GUI usage.

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

$DEBUG = "0";

$cce->connectfd();

# Location of the drbd.conf file:
$cfgfile = '/etc/drbd.conf';

# Location of the drbd proc::
$drbdproc = '/proc/drbd';
#$drbdproc = '/tmp/drdbsync.txt';

# We are starting sane:
$bad_info_detected = '0';

# Parse data:
&main;

# Populate CCE:
&feedthemonster;

$cce->bye('SUCCESS');
exit(0);

##
# Subs:
##

sub main {

    open (F, $cfgfile) || die "Could not open $cfgfile: $!";
    while ($line = <F>) {
	chomp($line);
	next if $line =~ /^\s*$/; 		# skip blank lines
	if ($line =~ /degr-wfc-timeout (.*)\;/) {
    	    $degr_wfc_timeout = $1;
    	}
	if ($line =~ /on-disconnect (.*);/) {
    	    $on_disconnect = $1;
    	}
	if ($line =~ /on-io-error (.*);/) {
    	    $on_io_error = $1;
    	}
	if ($line =~ /rate (.*)M;/) {
    	    $rate = $1 . "M";
    	}
	if (($line =~ /on (.*) \{/) && (!$parsed_Std_Sec_fqdn)) {
    	    $parsed_Std_Pri_fqdn = $1;
    	}
	if (($line =~ /on (.*) \{/) && ($parsed_Std_Pri_fqdn)) {
    	    $parsed_Std_Sec_fqdn = $1;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Device)) {
    	    $parsed_Std_Pri_Device = "/dev/" . $2;
    	}
	if (($line =~ /device(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Device)) {
    	    $parsed_Std_Sec_Device = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Pri_Disk = "/dev/" . $2;
    	}
	if (($line =~ /disk(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Disk)) {
    	    $parsed_Std_Sec_Disk = "/dev/" . $2;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && (!$parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Pri_ipAddr = $1;
    	}
	if (($line =~ /address\s{0,10}(.*):/) && ($parsed_Std_Pri_ipAddr)) {
    	    $parsed_Std_Sec_ipAddr = $1;
    	}
    } 
    close(F);

    if ($DEBUG == "1") {
        print "degr_wfc_timeout	: $degr_wfc_timeout \n";
	print "on_disconnect		: $on_disconnect \n";
        print "rate			: $rate \n";
        print "parsed_Std_Pri_fqdn	: $parsed_Std_Pri_fqdn \n";
        print "parsed_Std_Sec_fqdn	: $parsed_Std_Sec_fqdn \n";
        print "parsed_Std_Pri_Device	: $parsed_Std_Pri_Device \n";
        print "parsed_Std_Sec_Device	: $parsed_Std_Sec_Device \n";
        print "parsed_Std_Pri_Disk	: $parsed_Std_Pri_Disk \n";
        print "parsed_Std_Sec_Disk	: $parsed_Std_Sec_Disk \n";
        print "parsed_Std_Pri_ipAddr	: $parsed_Std_Pri_ipAddr \n";
        print "parsed_Std_Sec_ipAddr	: $parsed_Std_Sec_ipAddr \n";
    }
    
    open (F, $drbdproc) || die "Could not open $drbdproc: $!";
    while ($line = <F>) {
	chomp($line);
	if ($DEBUG == "1") {
	    print $line . "\n";
	}
	next if $line =~ /^\s*$/; 		# skip blank lines
	if ($line =~ / 0: cs:(.*) st:(.*) ld:(.*)/) {
    	    $ConStatus = $1;
    	    $SyncStatus = $2;
    	    $LinkDiagnostic = $3;
    	}

        if ($line =~ /\[(.*)\] sync\'ed: (.*) \((.*)\/(.*)\)M/) {
            $sync_progress = "[" . $1 . "]";
            $sync_percentage = $2;
            $sync_current = $3;
            $sync_max = $4;
        }
        if ($line =~ /finish: (.*) speed: (.*)/) {
            $sync_finish = $1;
            $sync_speed = $2;
        }
    }
    close(F);
    if (($sync_progress == "") && ($sync_current == "") && ($sync_max == "") && ($sync_max == "")) {
	$synchronizing = "0";
    }
    else {
	$synchronizing = "1";
    }
    if ($DEBUG == "1") {
        print "\n";
        print "ConStatus		: $ConStatus \n";
        print "SyncStatus		: $SyncStatus \n";
        print "LinkDiagnostic		: $LinkDiagnostic \n";
        print "sync_progress		: $sync_progress \n";
        print "sync_percentage		: $sync_percentage \n";
        print "sync_current		: $sync_current \n";
        print "sync_max		: $sync_max \n";
        print "sync_finish		: $sync_finish \n";
        print "sync_speed		: $sync_speed \n";
        print "synchronizing		: $synchronizing \n";
    }

}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
	&validate;
	# This is where we actually populate CCE with the gathered info:

        @oids = $cce->find('Cluster');
        if ($#oids < 0) {
		if ($DEBUG == "1") {
	                print "Object not yet in CCE. Creating new one.\n";
		}
                ($ok) = $cce->create('Cluster', {
                'degr_wfc_timeout' => $degr_wfc_timeout,
                'on_disconnect' => $on_disconnect,
                'on_io_error' => $on_io_error,
                'rate' => $rate,
                'Std_Pri_fqdn' => $parsed_Std_Pri_fqdn,
                'Std_Pri_ipAddr' => $parsed_Std_Pri_ipAddr,
                'Std_Pri_Device' => $parsed_Std_Pri_Device,
                'Std_Pri_Disk' => $parsed_Std_Pri_Disk,
                'Std_Sec_fqdn' => $parsed_Std_Sec_fqdn,
                'Std_Sec_ipAddr' => $parsed_Std_Sec_ipAddr,
                'Std_Sec_Device' => $parsed_Std_Sec_Device,
                'Std_Sec_Disk' => $parsed_Std_Sec_Disk,
                'ConStatus' => $ConStatus,
                'SyncStatus' => $SyncStatus,
                'LinkDiagnostic' => $LinkDiagnostic,
                'sync_progress' => $sync_progress,
                'sync_percentage' => $sync_percentage,
                'sync_current' => $sync_current,
                'sync_max' => $sync_max,
                'sync_finish' => $sync_finish,
                'sync_speed' => $sync_speed,
                'synchronizing' => $synchronizing,
                'timestamp' => time()
                });
        }
        else {
		if ($DEBUG == "1") {
	        	print "Object already present in CCE. Updating it.\n";
		}
                # find the system object
                ($sys_oid) = $cce->find('Cluster');
                ($ok, $sys) = $cce->get($sys_oid);
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'degr_wfc_timeout' => $degr_wfc_timeout,
                'on_disconnect' => $on_disconnect,
                'on_io_error' => $on_io_error,
                'rate' => $rate,
                'Std_Pri_fqdn' => $parsed_Std_Pri_fqdn,
                'Std_Pri_ipAddr' => $parsed_Std_Pri_ipAddr,
                'Std_Pri_Device' => $parsed_Std_Pri_Device,
                'Std_Pri_Disk' => $parsed_Std_Pri_Disk,
                'Std_Sec_fqdn' => $parsed_Std_Sec_fqdn,
                'Std_Sec_ipAddr' => $parsed_Std_Sec_ipAddr,
                'Std_Sec_Device' => $parsed_Std_Sec_Device,
                'Std_Sec_Disk' => $parsed_Std_Sec_Disk,
                'ConStatus' => $ConStatus,
                'SyncStatus' => $SyncStatus,
                'LinkDiagnostic' => $LinkDiagnostic,
                'sync_progress' => $sync_progress,
                'sync_percentage' => $sync_percentage,
                'sync_current' => $sync_current,
                'sync_max' => $sync_max,
                'sync_finish' => $sync_finish,
                'sync_speed' => $sync_speed,
                'synchronizing' => $synchronizing,
                'timestamp' => time()
                });

	}

}

# Validate the settings before we pass them on to CCE:
sub validate {

    if (($degr_wfc_timeout ne "") && ($on_disconnect ne "") && ($rate ne "") && ($parsed_Std_Pri_fqdn ne "") &&
        ($parsed_Std_Sec_fqdn ne "") && ($parsed_Std_Pri_Device ne "") && ($parsed_Std_Sec_Device ne "") &&
        ($parsed_Std_Pri_Disk ne "") && ($parsed_Std_Sec_Disk ne "") && ($parsed_Std_Pri_ipAddr ne "") && 
        ($parsed_Std_Sec_ipAddr ne "")) {
	$firstblock = "ok";
    }
    if (($ConStatus ne "") && ($SyncStatus ne "") && ($LinkDiagnostic ne "")) {
	$secondblock = "ok";
	}
    if (($LinkDiagnostic ne "Consistent") && ($synchronizing == "1") && ($sync_progress ne "") && ($sync_percentage ne "") &&
        ($sync_current ne "") && ($sync_max ne "") && ($sync_finish ne "") && ($sync_speed ne "")) {
	$thirdblock = "ok";
    }
    if (($LinkDiagnostic == "Consistent") && ($synchronizing == "0")) {
	$thirdblock = "ok";
    }

    if ($DEBUG == "1") {
	print "1: $firstblock\n";
	print "2: $secondblock\n";
	print "3: $thirdblock\n";
    }
    
    if (($firstblock ne "ok") || ($secondblock ne "ok") || ($thirdblock ne "ok")) {
	print STDERR "update_cluster_info.pl reports inconsistent data! \n";
	exit(1);
    }
}

$cce->bye('SUCCESS');
exit(0);

